#!/usr/bin/perl

use FormParse;
use Date::Manip;

#
# This CGI script is the interface used to 
# set the daily ad allocation on each web site.
#
# Notes:
#	- ad_info.dat = properties of each ad
#	- ad_x_alloc.dat = unbalanced large ad allocations
#	- ad_x_rotate.dat = final rotation list for large banner 
#			 (generated each hour by ad_rotateset.pl)
#	- surviving add list is sorted by AdID
#	- removed TABLEs to make page display faster
#	- Added 'Stop on Target' and 'Stop on Date' columns
#	- Reads and writes ad_x_alloc.dat
#	- Shows projected inventory (page impressions)
#	- Ad position set by changing adposition variable
#	- adinfo.dat changed to ad_info.dat
#	- Fixed American date input to ISO
#	- When totalling impressions allocated, only count ads within current date range
#
#
# Modified:  Kurt -- Fri Mar 10 11:06:01 2000


require 'ad_config_lib.pl';	# Site-specific settings for adsystem scripts


# Ad system settings


	$backcolor = "#DDFFFF";
	$mod_date = "06.01.2000";
	$servername = $ENV{'SERVER_NAME'};	# example: $servername = www.davecentral.com


# Read ad position from QUERY_STRING or default to position 1

	$adposition = $1 if $ENV{'QUERY_STRING'} =~ /^(\d)/;
	$adposition ||= 1;
	$thisscript = "/cgi-bin/ad_allocedit.cgi\?$adposition";	
	$allocfile = 'ad_' . $adposition . '_alloc.dat';
	$largead_position = 1 if ( grep ( /$adposition/, ( 1, 5 ) ) );	# Set adposition list of positions that can place run large banners

	%adtypes = ( 1, 'Premium Large',
		    2, 'Display',
		    3, 'Upper Small',
		    4, 'Bottom Small',
		    5, 'Bottom Large');

	$adtype = $adtypes{$adposition};


# Figure out the date and find most recent summary log files for this web site 

	$yesterday = &Date::Manip::UnixDate("yesterday", '%y%m%d');
	$today = &Date::Manip::UnixDate("today", '%y%m%d');
	$todayfullsysdate = &Date::Manip::UnixDate("today", '%Y%m%d');	
	$yesterday_adlog = $weblogpath . '/ads_' . $yesterday . '.log';		# Path where to find log of yesterdays impressions per ad
	$pagetrafficlog = $weblogpath . '/visitors' . $today . '.log';		# Define to estimate today's available page impressions


# Dump out here if we're just debugging 

	&DumpAdConfig if $debugging == 1;



# Read CGI inputs and check user access

	$form = new FormParse;
	$form->Parse;		

	print "Content-type: text/html \n\n";
	print "<HTML><HEAD><TITLE>$servername $adtype Ad Allocation</TITLE></HEAD>\n";
	print "<BODY BGCOLOR=\"$backcolor\">\n";

        $password = $1 if $form->{'in'}->{'Password'} =~ /(\w{2,8})/;

        if (($force_login eq 'Y') && ($password ne $adpasswd)) {
	    print "<BR><B>Login incorrect.</B> Please login via \
	    <A HREF=\"$mainmenu\">ad system main menu page</A>. \
	    </BODY></HTML>";
	    exit;
        }




# Read ad impressions delivered to each ad yesterday

	if ( open ( INFILE, "< $yesterday_adlog" ) ) {
	    while (<INFILE>) {
		($adid,$impr,$cliks,$site) = split(/\t/, $_);
		$ad_impressions_yesterday{$adid} = $impr;
	    }
	    close INFILE;
	} else {
	    $errors .= "Couldn't read $yesterday_adlog: $! <BR> \n";
	}
	


# Determine how much inventory will be available today


	if ( -e "$pagetrafficlog" ) {

		$hour = `/bin/date +%H`;
		$hour =~ s/^0//;

		if ($hour > 1) {
	
			if ( open(INFILE, "< $pagetrafficlog" )) {
		    		while (<INFILE>) {
				($dt,$vs,$pgs,$st) = split(',', $_);
				$pages += $pgs;
		    	}
		    	close INFILE;
			} else {
		    		$errors .= "Couldn't read $pagetrafficlog: $! <BR> \n";
			}
		
			$available_inventory = int($pages * ( 24 / $hour));
		}
		$available_inventory = 'Too soon to tell.' if (!$available_inventory);
		
		$available_inventory = "Projected Page Impressions:\t\t<B>$available_inventory</B>\n";

	}





# Build a hash of ads available in ad_info.dat for this ad position

	if ( open( ADINFILE, "< $datfilepath/ad_info.dat" ) ) {
	
	    while (<ADINFILE>) {
	    chomp;
	
		($AdID,$ht,$wh,$tt,$at,$re) = split(/\t/,$_);
			
		if ( $largead_position ) {
	    	    next if $wh < 380;
		} else {
	    	    next if $wh > 140;
		}
		
		
		if ( $can_run_megabanners ne 'Y') {
		    if ( (substr($AdID, 4, 1)) eq '6' ) {
			$errors .= "Skipping $AdID since this site can not run megabanners.\n" if ($debugging == 1);
			next;
		    }
		}
		
		
		if ( $verify_gifs_exists eq 'Y' ) {

		    $gif_filename = $gif_filepath . $AdID . ".gif";
		
		    if ( !( -e $gif_filename )) {
			$gif_errors .= "$gif_filename does not exist -- please remove $AdID from ad_info.dat\n";
			next;
	  	    }
	
		}	
		
		
		$adids{$AdID} = $AdID;
		
	    }
	    close ADINFILE;

	    @adids2 = (sort keys %adids);
	
	} else {
	
	    $errors .= "Couldn't read $datfilepath/ad_info.dat: $! <BR> \n";
	
	}
	
	
# Print page top

	print <<TOP_OF_PAGE;

	<FONT FACE="ARIAL,HELVETICA" SIZE=4 COLOR=RED>
	<B><I>mod_adbanners Control System v$mod_date</I></B></FONT>
	<BLOCKQUOTE>
	<FONT FACE="ARIAL,HELVETICA" SIZE=6><B>$servername $adtype Ad Rotation:</B></FONT>
	<PRE>
$gif_errors
	
<FORM METHOD="POST" ACTION="$thisscript">
<B>Kill	AdID		DailyTarget  StopOnTarget  StartDate	     StopDate	   Impressions Delivered Yesterday</B>
TOP_OF_PAGE



# Process input from previous web page 


	$writeincount_adl = $form->{'in'}->{'writeincount_adl'};

	if ($writeincount_adl =~ /\d/) {
	

# Make a list of surviving ads

		$Killed = 0;
 		$currentwritein = 1;

  		while ( $currentwritein <= $writeincount_adl ) {
			if (($form->{'in'}->{"Kill_$currentwritein"} eq 'T') || ($form->{'in'}->{"Impressions_$currentwritein"} < 10)) {
				$Killed++;				
			} else {
				$survivors{$form->{'in'}->{"AdID_$currentwritein"}} = $form->{'in'}->{"Impressions_$currentwritein"};
				$survivors{$form->{'in'}->{"AdID_$currentwritein"}} .= "\t" . $form->{'in'}->{"Target_$currentwritein"};
				$survivors{$form->{'in'}->{"AdID_$currentwritein"}} .= "\t" . $form->{'in'}->{"StartDate_$currentwritein"};
				$survivors{$form->{'in'}->{"AdID_$currentwritein"}} .= "\t" . $form->{'in'}->{"StopDate_$currentwritein"};
			}
			$currentwritein++; 
		}



# Add new ad to list of survivors

		$checknewads_idx = $writeincount_adl;
		for ( 1, 2 ) {
		    $checknewads_idx++;
		    if ( $form->{'in'}->{"Impressions_$checknewads_idx"} > 10 ) {
			$writeincount_adl++;
			$survivors{$form->{'in'}->{"AdID_$checknewads_idx"}} = $form->{'in'}->{"Impressions_$checknewads_idx"};
			$survivors{$form->{'in'}->{"AdID_$checknewads_idx"}} .= "\t" . $form->{'in'}->{"Target_$checknewads_idx"};
			$survivors{$form->{'in'}->{"AdID_$checknewads_idx"}} .= "\t" . $form->{'in'}->{"StartDate_$checknewads_idx"};
			$survivors{$form->{'in'}->{"AdID_$checknewads_idx"}} .= "\t" . $form->{'in'}->{"StopDate_$checknewads_idx"};
		    }
		}




# Print and save list of survivors

		$writeincount_adl = ( keys %survivors );
		$currentwritein = 0;


		foreach $AdID (sort keys %survivors) {

			($impressions, $targeted, $StartDate, $StopDate) = split(/\t/, $survivors{$AdID});

# Fix American date input to ISO
			
			$StartDate = &NiceDate2ISODate($StartDate) if $StartDate =~ /\d\/\d/;
			$StopDate = &NiceDate2ISODate($StopDate) if $StopDate =~ /\d\/\d/;

			$logdata .= "$AdID\t$impressions\t$targeted\t$StartDate\t$StopDate\n";

			$currentwritein++;
			
			# Only count active impressions within current date range
			
			if ( (!$StartDate) || $todayfullsysdate >= &ISODate2SysDate($StartDate) ) {
			    $errors .= "$AdID $todayfullsysdate >= $StartDate\n" if ($debugging == 1);
			    if ( ( !$StopDate ) ||  ( $todayfullsysdate < &ISODate2SysDate($StopDate) ) ) {
				$errors .= "$AdID is within active date range.\n\n" if ($debugging == 1);
				$total += $impressions;
			    } else {
				$errors .= "FALLTHRU B: $AdID $todayfullsysdate ! < $StopDate\n\n" if ($debugging == 1);
			    }
			} else {
			    $errors .= "FALLTHRU A: $AdID $todayfullsysdate ! >= $StartDate\n" if ($debugging == 1);
			}
			


			$targeted = 'CHECKED' if $targeted;

			print "<INPUT NAME=\"Kill_$currentwritein\" TYPE=\"checkbox\" VALUE=\"T\">  ";
			print "<SELECT NAME=\"AdID_$currentwritein\">";

			foreach $temp (@adids2) {
		 		print '<OPTION ';
		 		print 'SELECTED' if $temp eq $AdID;
				print '>' . $temp . "\n";
			}

			print "</SELECT>  ";
			print "<INPUT NAME=\"Impressions_$currentwritein\" SIZE=7 VALUE=\"$impressions\">\t  ";
			print "<INPUT NAME=\"Target_$currentwritein\" TYPE=\"CHECKBOX\" VALUE=\"T\" $targeted>\t";
			print "<INPUT NAME=\"StartDate_$currentwritein\" SIZE=11 VALUE=\"$StartDate\">\t";
			print "<INPUT NAME=\"StopDate_$currentwritein\" SIZE=11 VALUE=\"$StopDate\">\t";
			print $ad_impressions_yesterday{$AdID} . "\n";

		} # foreach survivor
	

# Create a new allocation file

		if ( open ( DATAFILE, "> $datfilepath/$allocfile" ) ) {
			print DATAFILE $logdata;
			close DATAFILE;

		} else {
		
		    $errors .= "Couldn't create/modify $datfilepath/$allocfile: $! <BR> \n";

		}
		
		
		

# Read data from allocation file

	} else {

		$currentwritein = 0;

		if ( open ( INFILE, "< $datfilepath/$allocfile" ) ) {
		    while (<INFILE>) {
		    chomp;
		    
			($AdID, $impressions, $targeted, $StartDate, $StopDate) = split( /\t/, $_ );

			$currentwritein++;
			
			
			# Only count active impressions within current date range
			
			if ( (!$StartDate) || $todayfullsysdate >= &ISODate2SysDate($StartDate) ) {
			    $errors .= "$AdID $todayfullsysdate >= $StartDate\n" if ($debugging == 1);
			    if ( ( !$StopDate ) ||  ( $todayfullsysdate < &ISODate2SysDate($StopDate) ) ) {
				$errors .= "$AdID is within active date range.\n\n" if ($debugging == 1);
				$total += $impressions;
			    } else {
				$errors .= "FALLTHRU B: $AdID $todayfullsysdate ! < $StopDate\n\n" if ($debugging == 1);
			    }
			} else {
			    $errors .= "FALLTHRU A: $AdID $todayfullsysdate ! >= $StartDate\n" if ($debugging == 1);
			}


			$targeted = 'CHECKED' if $targeted;

			print "<INPUT NAME=\"Kill_$currentwritein\" TYPE=\"checkbox\" VALUE=\"T\">  ";
			print "<SELECT NAME=\"AdID_$currentwritein\">";

			foreach $temp (@adids2) {
		 		print '<OPTION';
		 		if ( $temp eq $AdID ) {
				    print ' SELECTED';
				}
				print '>' . $temp . "\n";
			}

			print "</SELECT>  ";
			print "<INPUT NAME=\"Impressions_$currentwritein\" SIZE=7 VALUE=\"$impressions\">\t  ";
			print "<INPUT NAME=\"Target_$currentwritein\" TYPE=\"CHECKBOX\" VALUE=\"T\" $targeted>\t";
			print "<INPUT NAME=\"StartDate_$currentwritein\" SIZE=11 VALUE=\"$StartDate\">\t";
			print "<INPUT NAME=\"StopDate_$currentwritein\" SIZE=11 VALUE=\"$StopDate\">\t";
                        print $ad_impressions_yesterday{$AdID} . "\n";

		    }  # while INFILE
		    close INFILE;
		    
		    
		} else {
			$errors .= "Can't read $datfilepath/$allocfile: $! <BR>";
		}
		
		

		$writeincount_adl = $currentwritein;

	}  # endif $writeincount_adl



	$writeincount_adl2 = $writeincount_adl + 1;
	$writeincount_adl3 = $writeincount_adl + 2;

	foreach $temp (@adids2) {
		$selectlist .= "<OPTION>$temp\n";
	}


	$errors = "<FONT COLOR=RED><H3>Errors occured during processing!</H3\>$errors</FONT>" if $errors;


# Print rest of page

	print <<BOTTOM;

$errors
Total impressions allocated for today:	<B>$total</B>
$available_inventory
<B>$writeincount_adl</B> $adtype Ads on $servername

	<B>AdID		DailyTarget  	StopOnTarget  StartDate	     	   StopDate</B>
New:	<SELECT NAME="AdID_$writeincount_adl2"> $selectlist </SELECT>  <INPUT NAME="Impressions_$writeincount_adl2" SIZE=7>	  <INPUT NAME="Target_$writeincount_adl2" TYPE="CHECKBOX" VALUE="T">	<INPUT NAME="StartDate_$writeincount_adl2" SIZE=11>	<INPUT NAME="StopDate_$writeincount_adl2" SIZE=11>
New:	<SELECT NAME="AdID_$writeincount_adl3"> $selectlist </SELECT>  <INPUT NAME="Impressions_$writeincount_adl3" SIZE=7>	  <INPUT NAME="Target_$writeincount_adl3" TYPE="CHECKBOX" VALUE="T">	<INPUT NAME="StartDate_$writeincount_adl3" SIZE=11>	<INPUT NAME="StopDate_$writeincount_adl3" SIZE=11>
</PRE>
	<INPUT TYPE="HIDDEN" NAME="writeincount_adl" VALUE="$writeincount_adl"> 
	<INPUT TYPE="HIDDEN" NAME="Password" VALUE="$password">
<INPUT TYPE="SUBMIT" VALUE="Save Changes to $allocfile on $servername">
	</FORM><FORM METHOD=POST ACTION="$mainmenu">
	<INPUT TYPE="HIDDEN" NAME="Password" VALUE="$password">
<INPUT TYPE="SUBMIT" VALUE="Return to Main Menu">
	</FORM>

	</BLOCKQUOTE>
	</BODY>
	</HTML>

BOTTOM




sub NiceDate2ISODate {

        local ($nicedate) = @_;
        ($mon,$day,$year) = split(/[\/\-]/, $nicedate);
        $day =~ s/^0//;
        $day = '0' . $day if $day < 10;
        $mon =~ s/^0//;
        %monthlist = qw/1 Jan 2 Feb 3 Mar 4 Apr 5 May 6 Jun 7 Jul 8 Aug 9 Sep 10 Oct 11 Nov 12 Dec/;
        $year = `/bin/date +%Y` unless $year =~ /\d\d\d\d/;
        $year =~ s/\W//g;
        
        return $day . '/' . $monthlist{$mon} . '/' . $year;

}  # Converts 2/03/98 to 03/Mar/1998



sub ISODate2SysDate {

        local ($ISODate) = @_;
        if ($ISODate =~ /(\d+)\/(\w+)\/(\d+)/) {
                %monthlist = qw/Jan 01 Feb 02 Mar 03 Apr 04 May 05 Jun 06 Jul 07 Aug 08 Sep 09 Oct 10 Nov 11 Dec 12/;
                return $3 . $monthlist{$2} . $1;
        } else {
                return 0;
        } 

}  # Converts 02/Jan/1998 to 19980102 to do date comparisons



__END__


