#!/usr/local/bin/perl

use Date::Manip;

#
# This shell script runs once every hour to log
# all ad stats so far today then adjusts new ad
# rotation lists according to ad_x_alloc.dat.
#
# Notes:
#	- Data beyond 50 clicks from a single IP is ignored (assumed to be a robot)
#	- Ads are stopped for rest of day if StopOnTarget 
#	- Ads are stopped on given StopDate
#	- Added command line option to generate yesterday's log file
#	- Creates a lean version of ad_info.dat (ad_info_lite.dat)
#       - Commented out the ErrorDocument line to allow http.conf directive to work. (Marc)
#
# Modified:  Kurt -- Thu Mar 23 09:37:29 EST 2000	Don't process blank adids

require 'ad_config_lib.pl';
	

# Determine the datestamp of the output summary log files

	$todayfullsysdate = &Date::Manip::UnixDate("today", '%Y%m%d');	# Comparative date for placing time-limited ads
	
	# Command line argument 'yesterday' means "recount yesterday's ad stats"
	
	if ($ARGV[0] =~ 'yesterday') {
		$logdate = &Date::Manip::UnixDate("yesterday", '%y%m%d');
		$weblogfile = $previous_weblogfile;
		$dont_set_adrotations = 1;
	} elsif ($ARGV[0] =~ /(\d+)/) {
		$logdate = $1;
		$weblogfile = $previous_weblogfile;
		$dont_set_adrotations = 1;
	} else {
                $logdate = &Date::Manip::UnixDate("today", '%y%m%d');
	}


	$adlogfile = "$weblogpath/ads_" . $logdate . '.log';		# Name of ad stats summary log



	&DumpAdConfig if $debugging == 1;



# Get today's ad stats for this web site

	&GatherStats;




# Save today's ad stats to a log file

	if ( open ( OUTFILE, "> $adlogfile" ) ) {
		foreach $AdID (sort keys %AdImpressions) {
			next if !$AdData{$AdID.'IMPRESSIONS'};
			print OUTFILE "$AdID\t" . $AdData{$AdID.'IMPRESSIONS'}  . "\t" . $AdData{$AdID.'CLICKS'} . "\t$siteid\n";
		}
		close OUTFILE;
	} else {
		print "Couldn't create $adlogfile: $!";
	}


	%AdData = ();


# Stop here if logging yesterday or this is www.postcardmaker.com (gets ad rotation list from www.animfactory.com)

	exit if $dont_set_adrotations;
	

# Make new rotation file for each ad position

	foreach $adposition (@positions_list) {
		&MakeRotateFile("$datfilepath/ad_" . $adposition . "_alloc.dat", "$datfilepath/ad_" . $adposition . "_rotate.dat");
	}



# Generate reserved pages file ad_reservations.dat by reading ad_reservations_alloc.dat

	if ( open (INFILE, "< $datfilepath/ad_reservations_alloc.dat" ) ) {
		while (<INFILE>) {
		    chomp;
		    
		    ($pagename, $position, $AdID, $StartDate, $StopDate) = split(/\t/, $_);
		    
		    next if $AdID !~ /\w+/;
		    
		    if ( $StartDate ) {
			if ( $todayfullsysdate <= &ISODate2SysDate($StartDate) ) {
			    print "Reseved $pagename, $position, $AdID doesn't start yet ( $todayfullsysdate <= $StartDate ) \n" if ( $debugging == 1 );
			    next;
			}
		    }
		    
		    if ( $StopDate ) {
			if ( $todayfullsysdate >= &ISODate2SysDate($StopDate) ) {
			    print "Reseved $pagename, $position, $AdID was stopped on date ( $todayfullsysdate >= $StopDate ) \n" if ( $debugging == 1 );
			    next;
			}
		    }
		    
		    $CurrentAdsList{$AdID} = 1;
		    
		    $reserved_filedata .= "$pagename\t$position\t$AdID\n";
		    
		}
		close INFILE;
		
	} # if ad_reservations_alloc.dat




# Save the new ad_reservations.dat file

	if ( open( OUTFILE, "> $datfilepath/ad_reservations.dat") ) {
	    print OUTFILE $reserved_filedata;
	    close OUTFILE;
	} else {
	    print "Could not recreate $datfilepath/ad_reservations.dat : $! \n";
	}





# Grab the lines from ad_info.dat of the ads in that are in the new rotation list


	if ( open (INFILE, "< $datfilepath/ad_info.dat" ) ) {
		while (<INFILE>) {
			next if $_ !~ /^(\w+)\t/;
			$adinfodata .= $_ if $CurrentAdsList{$1};
		}
		close INFILE;
	} else {
		print "Couldn't read $datfilepath/ad_info.dat: $! \n";
	}



# Make a lean and mean ad_info_lite.dat file listing only current ads


	if ( $adinfodata =~ /\w+/ ) {
	    if ( open (OUTFILE, "> $datfilepath/$adinfolite" ) ) {
		print OUTFILE $adinfodata;
		close OUTFILE;
	    } else {
		print "Can't create $datfilepath/$adinfolite: $! \n";
	    }
	} else {
	    die "$0 stopped for $sitename because ad_info.dat may either be missing or empty";
	}



# Create an .htaccess file to 403 deny obnoxious ad clicking robots


    if ( $clicklimit > 0 ) {

	for $remote_host (keys %ClicksPerVisitor) {
		if ($ClicksPerVisitor{$remote_host} > $clicklimit) {
			$robotlist .= "deny from $remote_host\n";
		}		
	}

	if ( open(OUTFILE, "> $datfilepath/.htaccess") ) {
		print OUTFILE "ErrorDocument 404 $errordocument_404\n" if $errordocument_404;
		print OUTFILE $robotlist;
		close OUTFILE;
		if ($htdocs_path) {
		    $command = `/bin/cp $datfilepath/.htaccess $datfilepath/$htdocs_path/.htaccess`;
		}
	} else {
		print "Can't create $datfilepath/.htaccess: $! ";
	}

    }  # if clicklimit
    



# Extract ad requests from access_log(s)

sub GatherStats {



	if ( open(INFILE, "< $weblogfile") ) {

		while (<INFILE>) {

			next if $_ !~ /^(\S+) \- \- \[.+\] \"([A-Z])+ (\S+) \S+\" [23]/;

			$remote_host = $1;
			next if $2 eq 'HEAD';
			$request = $3;

			next if ($ClicksPerVisitor{$remote_host} > $clicklimit);

			if ($request =~ /$bannerpath(\w{10})\.gif/) {
				$AdImpressions{$1}++;
				$AdData{$1.'IMPRESSIONS'}++;
			} elsif ($request =~ /$clickscript\?([\w\.\-]*),(\w{10})/) {
				$AdData{$2.'CLICKS'}++;
				$ClicksPerVisitor{$remote_host}++;
			} else {
				next;
			}

		}  # while access_log
		close INFILE;

	} else {

		print "Can't read $weblogfile: $!\n";

	}


 }  # sub GatherStats



sub MakeRotateFile {

	local( $allocfile, $rotatefile ) = @_;


	$lowest_allocation = 99999999;
	%AdList = ();
	@results_list = ();
	$results = '';
	$results2 = '';

	print "\n\nGenerating rotation list for ad position $adposition \n\n" if ( $debugging == 1 );
	

# Read current ad allocation for this ad position



	if ( open(INFILE, "< $allocfile") ) {
	    while (<INFILE>) {
	    chop;
		($AdID, $allocation, $targeted, $StartDate, $StopDate) = split(/\t/, $_);
		next if $AdID !~ /\w{10}/;
		next if $allocation < 10;
		next if ( ( $targeted ) && ( $AdImpressions{$AdID} > $allocation ) );
		if ( $StartDate ) {
		    if ( $todayfullsysdate <= &ISODate2SysDate($StartDate) ) {
			print "$AdID doesn't start yet ( $todayfullsysdate <= $StartDate ) \n" if ( $debugging == 1 );
			next;
		    }
		}
		if ( $StopDate ) {
		    if ( $todayfullsysdate >= &ISODate2SysDate($StopDate) ) {
			print "$AdID was stopped on date ( $todayfullsysdate >= $StopDate ) \n" if ( $debugging == 1 );
			next;
		    }
		}
		$AdList{$AdID} = $allocation;
		print "$AdID allocated for $allocation impressions...\n" if ( $debugging == 1 );
		$results2 .= "$AdID\n";
		$lowest_allocation = $allocation if $allocation < $lowest_allocation;
	    }
	    close INFILE;
	} else {
	    print "Couldn't read $allocfile: $! \n";
	    return;
	}
	

	print "Lowest allocation is $lowest_allocation\n\n" if ( $debugging == 1 );


# Create the new rotation list for this ad position

	foreach $AdID (keys %AdList) {
		$repeat = int( $AdList{$AdID} / $lowest_allocation );
		print "$AdID repeat factor = $repeat\n" if ( $debugging == 1 );
		push(@results_list, $AdID) while ($repeat-- > 0);
		$CurrentAdsList{$AdID} = 1;
	}


	$results = join("\n", (&ShuffleList(@results_list)) );
	$results .= "\n";


# Default display ad to anda5050en if display ad (position 2) rotation burns out

	if (($rotatefile eq 'ad_2_rotate.dat') && ($results !~ /\w+/)) {
		$results = "anda5050en\n";
	}



# Save new rotation list file

	if (open(OUTFILE, "> $rotatefile")) {
	    print OUTFILE $results;
	    print OUTFILE $results2 if ($results !~ /\w+/);
	    close OUTFILE;
	} else {
	    print "Couldn't create $rotatefile: $! \n";
	}
	
	

 } # sub MakeRotateFile




sub ISODate2SysDate {

        local ($ISODate) = @_;
        if ($ISODate =~ /(\d+)\/(\w+)\/(\d+)/) {
                %monthlist = qw/Jan 01 Feb 02 Mar 03 Apr 04 May 05 Jun 06 Jul 07 Aug 08 Sep 09 Oct 10 Nov 11 Dec 12/;
                return $3 . $monthlist{$2} . $1;
        } else {
                return 0;
        } 

}  # Converts 02/Jan/1998 to 19980102 to do date comparisons





sub ShuffleList {

    my @result = @_;
    my $num_items = @result;
    while ($num_items > 1) {
	my $rand_item = rand $num_items;
	$num_items--;
	@result[$rand_item, $num_items] = @result[$num_items, $rand_item];
    }
    
    @result;

}  # sub ShuffleList randomly shuffles a list, straight out of "Effective Perl Programming", page 100 (item 47)




__END__
